@@ -17,7 +17,7 @@
 #
 '''Facilities for handling the D-BUS driven configuration backend.'''
 
-import dbus, dbus.service, gobject, shutil
+import dbus, dbus.service, gobject, logging, shutil
 import errno, os, os.path, re, pty, signal, tempfile
 
 from gettext               import gettext as _
@@ -64,11 +64,15 @@ class PolicyKitService(dbus.service.Object):
             if sender:
                 kit = dbus.SystemBus().get_object('org.freedesktop.PolicyKit', '/')
                 kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit')
-                pid = dbus.UInt32(os.getpid())
 
-                if 'yes' != kit.IsProcessAuthorized(action, pid, False):
-                    raise AccessDeniedException('Process not authorized by PolicyKit')
-                if 'yes' != kit.IsSystemBusNameAuthorized(action, sender, False):
+                # Note that we don't use IsProcessAuthorized because we have
+                # no ways to get the PID of the front-end, so we're left
+                # with checking that its bus name is authorised instead
+		# See http://bugzilla.gnome.org/show_bug.cgi?id=540912
+                granted = kit.IsSystemBusNameAuthorized(action, sender, False)
+                logging.info('authorization of system bus name \'%s\': %r', sender, granted)
+
+                if 'yes' != granted:
                     raise AccessDeniedException('Session not authorized by PolicyKit')
 
         except AccessDeniedException:
@@ -80,7 +84,7 @@ class PolicyKitService(dbus.service.Object):
 class ExternalToolDriver(PolicyKitService):
     '''
     A D-BUS service which mainly is implemented
-    as wrapper arround some external program.
+    as wrapper around some external program.
     '''
 
     # pylint: disable-msg=C0103,E0602
@@ -93,8 +97,9 @@ class ExternalToolDriver(PolicyKitService):
         self.__line_buffer = ''
         self.__pid = -1
         self.__fd = -1
+        self._hup_expected = False
 
-    def __spawn_external_tool(self):
+    def _spawn_external_tool(self):
         '''Launches the external tool backing this service.'''
 
         pid, fd = pty.fork()
@@ -104,7 +109,6 @@ class ExternalToolDriver(PolicyKitService):
             assert False # should not be reached
 
         os.waitpid(pid, os.P_NOWAIT)
-
         return pid, fd
 
     def __io_handler(self, fd, condition):
@@ -153,11 +157,13 @@ class ExternalToolDriver(PolicyKitService):
         self._check_permission(sender)
 
         self.__line_buffer = ''
-        self.__pid, self.__fd = self.__spawn_external_tool()
+        self.__pid, self.__fd = self._spawn_external_tool()
 
-        gobject.io_add_watch(self.__fd,
-                             gobject.IO_IN | gobject.IO_HUP,
-                             self.__io_handler)
+        if -1 != self.__fd:
+            self._hup_expected = False
+            gobject.io_add_watch(self.__fd,
+                                 gobject.IO_IN | gobject.IO_HUP,
+                                 self.__io_handler)
 
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='', out_signature='',
@@ -239,6 +245,7 @@ class IrRecordDriver(ExternalToolDriver):
     _token_next_key         = 'Please enter the name for the next button'
     _token_wait_toggle_mask = 'If you can\'t see any dots appear'
     _token_finished         = 'Successfully written config file.'
+    _token_almost_finished  = 'Creating config file in raw mode.' # Shown when _token_finished should not be expected.
 
     # Last instance index for automatic object path creation:
     __last_instance = 0
@@ -262,6 +269,7 @@ class IrRecordDriver(ExternalToolDriver):
             self._cmdargs.append(filename)
 
         super(IrRecordDriver, self).__init__(connection, path)
+        logging.info("__init__.IrRecordDriver(): irrecord cmdargs=%s", self._cmdargs)
 
     def _on_run_external_tool(self):
         '''Enters to the working directory and executes irrecord.'''
@@ -289,7 +297,15 @@ class IrRecordDriver(ExternalToolDriver):
     def _on_hangup(self):
         '''Shutdown the driver, when irrecord terminates unexpectedly.'''
 
-        if list(self.locations):
+        if(self._hup_expected):
+            # This was expected. It's a success:
+            filename = os.path.join(self._workdir, self._filename)
+            configuration = open(filename).read()
+            self.ReportSuccess(configuration)
+            self.Release()
+
+        if list(self.locations): #self.locations is in the dbus base class. This check prevents a shutdown when the dbus service should be kept alive.
+            # irrecord stopped when we were expecting more interaction:
             self.ReportFailure(_('Custom remote control configuration aborted unexpectedly.'))
             self.Release()
 
@@ -314,6 +330,7 @@ class DetectParametersDriver(IrRecordDriver):
     def __init__(self, connection, driver, device):
         super(DetectParametersDriver, self).__init__(connection, driver, device)
         self.__report_progress = False
+        self._hup_expected = False
 
     def _prepare_workdir(self):
         '''Removes the configuration file from working directory when needed.'''
@@ -344,13 +361,16 @@ class DetectParametersDriver(IrRecordDriver):
             return
 
         if line.startswith(self._token_random_buttons):
+            # TODO: The talk of "steps" here does not make much sense
+            # in terms of a GTK+ progress bar.
+
             self.RequestAction(
                 _('Press random buttons on your remote control.'),
-                _('It is very important that you press many different ' +
-                  'buttons and hold them down for approximately one ' +
-                  'second. Each button should move the progress bar ' +
-                  'by at least one, but in no case by more than ten ' +
-                  'steps.'))
+                _('When you press the Start button, it is very important '
+                  'that you press many different buttons and hold them down ' +
+                  'for approximately one second. Each button should move the ' +
+                  'progress bar by at least one step, but in no case by more ' +
+                  'than ten steps.'))
 
             return
 
@@ -374,6 +394,15 @@ class DetectParametersDriver(IrRecordDriver):
             self.Release()
             return
 
+        if line.startswith(self._token_almost_finished):
+            # For some remotes (such as the XBox replacement remote, 
+            # with the StreamZap receiver, there is no additional output after 
+            # pressing Enter to stop the key-code learning, 
+            # so _token_finished is never seen,
+            # but in that case we see this message instead beforehand.
+            self._hup_expected = True
+            return
+
     def _on_next_chunk(self, chunk):
         '''
         Identifies progress indications in the next chunk of irrecord output,
@@ -443,19 +472,9 @@ class LearnKeyCodeDriver(IrRecordDriver):
 
         return None
 
-    def _on_run_external_tool(self):
-        '''Runs irrecord with --resume switch when supported.'''
-        irrecord = os.popen('%s --help' % config.LIRC_IRRECORD)
-
-        if -1 == irrecord.read().find('--resume'):
-            self.ReportFailure(_(
-                'The installed lirc package doesn\'t support self-contained ' +
-                'key codes yet. Update to the latest lirc package if possible.'))
-
-        self._cmdargs.insert(1, '--resume')
-
-        super(LearnKeyCodeDriver, self)._on_run_external_tool()
-
+    def _spawn_external_tool(self):
+        '''Runs irrecord.'''
+        return super(LearnKeyCodeDriver, self)._spawn_external_tool()
 
 class BackendService(PolicyKitService):
     '''A D-Bus service that PolicyKit controls access to.'''
@@ -517,7 +536,7 @@ class BackendService(PolicyKitService):
         if self.connection.list_exported_child_objects('/'):
             return True
 
-        print 'Terminating %s due inactivity.' % self.SERVICE_NAME
+        print 'Terminating %s due to inactivity.' % self.SERVICE_NAME
         self.__loop.quit()
 
         return False
@@ -546,45 +565,46 @@ class BackendService(PolicyKitService):
         if not os.path.isfile(oldfile):
             raise UnsupportedException('Cannot find %s script' % oldfile)
 
-        print 'Updating %s...' % oldfile
+        logging.info('Updating %s...', oldfile)
+        logging.info('- receiver_values: %r', receiver_values)
+        logging.info('- remote_values: %r', remote_values)
 
         output = file(newfile, 'w')
         for line in file(oldfile, 'r'):
 
-            # Deal with lines starting with REMOTE_,
-            # replacing their values with ours:
-
+            # Identify directives starting with REMOTE_ and replacing their values with ours.
+            # Remove entry from the dict on match, so we know what was written.
             match = self.__re_remote_directive.match(line)
+            value = match and remote_values.pop(match.group(1), None)
 
-            if match:
-                # Remove entry from the dict, so we know what was written.
-                value = remote_values.pop(match.group(1), None)
-
-                if value is not None:
-                    print >> output, ('%s"%s"' % (match.group(0), value))
-                    continue
-
-            # Deal with lines starting with RECEIVER_,
-            # replacing their values with ours:
+            if value is not None:
+                logging.info('- writing %s"%s"', match.group(0), value)
+                print >> output, ('%s"%s"' % (match.group(0), value))
+                continue
 
+            # Identify directives starting with RECEIVER_ and replacing their values with ours.
+            # Remove entry from the dict on match, so we know what was written.
             match = self.__re_receiver_directive.match(line)
+            value = match and receiver_values.pop(match.group(1), None)
 
-            if match:
-                # Remove entry from the dict, so we know what was written.
-                value = receiver_values.pop(match.group(1), None)
-
-                if value is not None:
-                    print >> output, ('%s"%s"' % (match.group(0), value))
-                    continue
+            if value is not None:
+                logging.info('- writing %s"%s"', match.group(0), value)
+                print >> output, ('%s"%s"' % (match.group(0), value))
+                continue
 
             # Deal with the START_LIRCD line:
 
             match = self.__re_start_lircd.match(line)
 
             if match:
-                value = (start_lircd is None and 'true') or start_lircd
-                print >> output, (match.group(0) + value)
+                # pychecker says "Using a conditional statement with a constant value (true)",
+                # which is ridicilous, considering Python 2.4 doesn't have conditional statements
+                # yet (PEP 308, aka. 'true_value if condition else false_value') and the expression
+                # below ('condition and true_value or false_value') is the recommended aquivalent.
+                value = (start_lircd is None) and 'true' or start_lircd
                 start_lircd = None
+
+                print >> output, (match.group(0) + value)
                 continue
 
             output.write(line)
@@ -666,17 +686,16 @@ class BackendService(PolicyKitService):
 
         if remote:
             values = {
-                'VENDOR': remote.vendor or _('Unknown'),
+                'VENDOR': remote.vendor or 'Unknown', # Note; We do not translate Unknown here, though we do in the UI.
                 'MODEL': remote.product or remote.name
             }
 
             self._write_hardware_configuration(remote_values=values)
 
         # Write remote configuration:
-        filename, include_needed = lirc.find_remote_config()
+        filename = lirc.find_remote_config()
 
-        if include_needed:
-            self.__write_include_statement(filename)
+        self.__write_include_statement(filename)
 
         print contents
 
@@ -688,10 +707,8 @@ class BackendService(PolicyKitService):
         '''Write include statement to central lircd.conf file.'''
 
         # read central lirc configuration file file:
-        lircd_config = os.path.join(config.LIRC_CONFDIR, 'lircd.conf')
-
         try:
-            contents = open(lircd_config).read()
+            contents = open(config.LIRC_DAEMON_CONF).read()
 
         except IOError:
             contents = ''
@@ -706,8 +723,8 @@ class BackendService(PolicyKitService):
             contents = ''
 
         # find existing include statement:
-        include_statement = 'include %s\n' % redirect
-        pattern = r'^\s*(#.*)?include\s+%s\s*$' % re.escape(redirect)
+        include_statement = 'include <%s>\n' % redirect
+        pattern = r'^\s*(#.*)?include\s+<%s>\s*$' % re.escape(redirect)
         pattern = re.compile(pattern, re.MULTILINE)
         match   = pattern.search(contents)
 
@@ -720,7 +737,7 @@ class BackendService(PolicyKitService):
                 contents += '#\n'
                 contents += '# Feel free to add any custom remotes to the configuration\n'
                 contents += '# via additional include directives or below the existing\n'
-                contents += '# Ubuntu include directives from your selected remote and/or\n'
+                contents += '# include directives from your selected remote and/or\n'
                 contents += '# transmitter.\n'
                 contents += '#\n'
 
@@ -737,7 +754,7 @@ class BackendService(PolicyKitService):
             contents = None
 
         if contents:
-            open(lircd_config, 'w').write(contents)
+            open(config.LIRC_DAEMON_CONF, 'w').write(contents)
 
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='', out_signature='',
@@ -822,6 +839,13 @@ def get_service(bus=None):
     return service
 
 if __name__ == '__main__':
+    # Support full tracing when --debug switch is passed:
+
+    from sys import argv
+
+    if '--debug' in argv or '-d' in argv:
+        logging.getLogger().setLevel(logging.NOTSET)
+
     # Integrate DBus with GLib main loops.
 
     from dbus.mainloop.glib import DBusGMainLoop
