@@ -27,7 +27,7 @@ import os, os.path
 
 from ConfigParser          import SafeConfigParser
 from gettext               import gettext as _
-from gnome_lirc_properties import lirc
+from gnome_lirc_properties import config, lirc
 
 HAL_SERVICE = 'org.freedesktop.Hal'
 HAL_MANAGER_PATH = '/org/freedesktop/Hal/Manager'
@@ -123,9 +123,18 @@ class HalDevice(object):
 
     def find_input_device(self):
         '''Find the Linux Input System device node associated with this device.'''
-        for device in self.find_children():
-            if device.get('info.category') == 'input':
-                return device['input.device']
+
+        if self.get('info.category') == 'input':
+            name = self.read_sysfs_file('..', 'name')
+
+            if name is not None:
+                return 'name=' + name.replace(' ', '?')
+
+            return self['input.device']
+
+        for child in self.find_children():
+            if child.get('info.category') == 'input':
+                return child.find_input_device()
 
         return None
 
@@ -141,13 +150,54 @@ class HalDevice(object):
 
         return None
 
+    def has_capability(self, capability):
+        '''Checks if the device has a certain capability...'''
+        return (capability in self.get('info.capabilities', []))
+
+    def get_sysfs_path(self, *path):
+        '''Returns the path for an associated sysfs entry.'''
+        return os.path.join(self['linux.sysfs_path'], *path)
+
+    def read_sysfs_file(self, *path):
+        '''Returns the content of an associated sysfs file.'''
+        sysfs_path = self.get_sysfs_path(*path)
+
+        if os.path.isfile(sysfs_path):
+            return open(sysfs_path).read().rstrip()
+
+        return None
+
+    def is_real_keyboard(self):
+        '''
+        Try figure out if this device is a keyboard. Keyboards are detected by
+        counting the number of supported keys. The original PC keyboard had 85
+        keys, so maybe this is a reasonable boundary. Maybe it would make more
+        sense to look for typical key-codes like SHIFT, CTRL, CAPSLOCK or
+        NUMLOCK?
+        '''
+
+        # check if HAL considers this device a keyboard:
+        if self.has_capability('input.keyboard'):
+            # read and parse key-code map from sysfs:
+            keys = self.read_sysfs_file('..', 'capabilities', 'key')
+
+            if keys is not None:
+                keys = [int(value, 16) for value in keys.split()]
+                keys = decode_bitmap(keys)
+
+                # check that at least 85 key-codes are supported:
+                if len(keys) >= 85:
+                    return True
+
+        return False
+
     def __str__(self):
         return self.__udi
     def __repr__(self):
         return '<HalDevice: %s>' % self.__udi
 
     # pylint: disable-msg=W0212
-    udi = property(lambda self: self.__udi)
+    udi = property(fget=lambda self: self.__udi)
 
 class HardwareDatabase(SafeConfigParser):
     '''Information about supported hardware.'''
@@ -230,13 +280,15 @@ class HardwareManager(gobject.GObject):
 
         device = self.lookup_device(udi)
 
-        if 'input.keyboard' in device.get('info.capabilities', []):
+        if (device.has_capability('input.keyboard') and
+            not device.is_real_keyboard()):
+
             product_name = str(device['info.product'])
             device_node = str(device['input.device'])
 
             properties = {
                 'compatible-remotes': 'linux-input-layer',
-                'lirc-driver': 'devinput',
+                'lirc-driver': config.LIRC_INPUT_DRIVER,
                 'device': device_node,
                 'udi': udi,
             }
@@ -400,7 +452,7 @@ class HardwareManager(gobject.GObject):
             # lookup the current device:
             device = self.lookup_device(device)
             product_name = device.get('info.product')
-            device_node = device.get('input.device')
+            device_node = device.find_input_device()
 
             # report search progress:
             self.__report_search_progress(product_name)
@@ -409,8 +461,8 @@ class HardwareManager(gobject.GObject):
             if product_name is None or device_node is None:
                 continue
 
-            # skip input devices that have the word "keyboard" in their product name:
-            if product_name.lower().find('keyboard') >= 0:
+            # skip input devices that seem to be keyboards:
+            if device.is_real_keyboard():
                 continue
 
             # report findings:
@@ -482,3 +534,14 @@ class HardwareManager(gobject.GObject):
         self.__search_canceled = True
 
     devinput_receivers = property(__get_devinput_receivers)
+
+def decode_bitmap(bits):
+    '''Decodes a bitmap as published in sysfs.'''
+
+    values, offset = list(), 0
+
+    for b in reversed(bits):
+        values += [offset + i for i in range(32) if b & (1 << i)]
+        offset += 32
+
+    return tuple(values)
